#
# Authors: Brian Skahan <bskahan@etria.com>
#          Tom von Schwerdtner <tvon@etria.com>
#
# Copyright 2004-2005, Etria, LLP
#
# This file is part of Quills
#
# Quills is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Quills is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Quills; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################

# Zope 2 imports
from AccessControl import ClassSecurityInfo
from DateTime import DateTime

# Archetypes imports (with optional Linguaplone integration)
try:
    from Products.LinguaPlone.public import *
except ImportError:
    from Products.Archetypes.public import *
from Products.Archetypes.Marshall import PrimaryFieldMarshaller
from Products.ATContentTypes.content.base import registerATCT

# CMF imports
from Products.CMFCore import permissions
from Products.CMFCore.utils import getToolByName

# Product imports
import MetaWeblogAPI
from interfaces import IWeblog
from config import PROJECTNAME, DEFAULT_LEFT_SLOTS, DEFAULT_RIGHT_SLOTS
from topic import Topic
import permissions as perms

# Zope 3 imports
from zope.interface import implements

WeblogSchema = BaseFolderSchema.copy() + Schema((

    # This is sort of cheating.  We are stealing the Dublin Core 'description'
    # field for our purposes, but then I don't seen any reason to duplicate the
    # information.
    TextField('description',
        searchable=1,
        accessor="Description",
        widget=TextAreaWidget(
            label='Description',
            description="""A brief description of this weblog.  This text will
            be displayed at the top of the page before any weblog entries.""",
            label_msgid="label_description",
            description_msgid="help_description",
            i18n_domain="plone"),
        ),

    ### 
    ## Configuration Options
    ### 
    BooleanField('onlyExcerptInWeblogView',
        default = False,
        widget=BooleanWidget(
                    label='Only show excerpts in overviews?',
                    description="""When enabled, show only the title and
                    excerpt in the main weblog view.  If an entry has no
                    excerpt, only its title will be displayed.""",
            ),
        schemata='configuration'
        ),

    BooleanField('groupByDates',
        default = True,
        widget=BooleanWidget(
                    label='Group entries by date in overviews?',
                    description="""When enabled, entries will be 
                    grouped under a header showing the
                    date. Otherwise, the entries will be just be shown
                    underneath eachother.
                    """,
            ),
        schemata='configuration'
        ),

    IntegerField('entriesPerPage',
        default = 20,
        widget=IntegerWidget(label='Entries Per Page',
            description="Select the number of weblog entries you would like \
                    to display on the front page and any other batched pages.",
            ),
        schemata='configuration'
        ),

    BooleanField('showTopicImagesInWeblogView',
        default = 1,
        widget=BooleanWidget(label='Show Topic Images In Weblog View?',
            description="This controls the display of topic images in the \
                    weblog view.",
            ),
        schemata='configuration'
        ),

    IntegerField('entriesPerPortlet',
        default = 5,
        widget=IntegerWidget(label='Entries Per Portlet',
            description="Select the number of entries you would like\
                    to display in the various Quills portlets.",
            ),
        schemata='configuration'
        ),

    BooleanField('trackbackEnabled',
        default = 1,
        widget=BooleanWidget(label='Enable the receiving of trackback pings?',),
        schemata='configuration'
        ),

    ),

    marshall=PrimaryFieldMarshaller(),
    )
# The subject is not language-specific
WeblogSchema['subject'].languageIndependent = True
#We do not force subject into a visible schema as in WeblogEntry.

class Weblog(BaseFolder):
    """Weblog object"""

    implements(IWeblog)

    schema = WeblogSchema
    security = ClassSecurityInfo()

    _properties = (
        {'id':'left_slots',  'type':'lines', 'mode':'wd'},
        {'id':'right_slots', 'type':'lines', 'mode':'wd'},
        )

    archetype_name = "Weblog"
    meta_type = 'Weblog'
    content_icon = 'weblog_icon.gif'
    allowed_content_types = (
            'WeblogEntry',
            'Folder', # To hold topic images.
            )

    _at_rename_after_creation = True

    default_view = 'weblog_view'
    immediate_view = 'weblog_view'
    actions = (
        { 'id': 'view',
          'name': 'View',
          'action': 'string:${folder_url}/',
          'permissions': (permissions.View,) },
        { 'id': 'topics',
          'name': 'Topics',
          'action': 'string:${folder_url}/topics/',
          'permissions': (permissions.View,) },
        { 'id': 'archive',
          'name': 'Archive',
          'action': 'string:${folder_url}/archive/',
          'permissions': (permissions.View,) },
          {
          'id': 'local_roles',
          'name': 'Sharing',
          'action': 'string:${object_url}/folder_localrole_form',
          'permissions': (permissions.ManageProperties,),
          },
        { 'id': 'drafts',
          'name': 'Drafts',
          'action': 'string:${folder_url}/weblogdrafts_view',
          'permissions': (permissions.ModifyPortalContent,) },
        { 'id': 'comment-manager',
          'name': 'Comments',
          'action': 'string:${folder_url}/manage_comments',
          'permissions': (permissions.ModifyPortalContent,) },
        )

    aliases = {
        '(Default)'  : 'weblog_view',
        'view'       : 'weblog_view',
        'index.html' : '',
        'edit'       : 'base_edit',
        'properties' : 'base_metadata',
        'sharing'    : 'folder_localrole_form',
        'gethtml'    : '',
        'mkdir'      : '',
        }

    def initializeArchetype(self, **kwargs):
        BaseFolder.initializeArchetype(self, **kwargs)

        # Setup the MetaWeblog API
        if not hasattr(self.aq_inner.aq_explicit, 'metaWeblog'):
            self.metaWeblog = MetaWeblogAPI.MetaWeblogAPI()
            # Alias metaWeblog to blogger so that performancing works.
            # See <http://performancing.com/node/3158> for details.
            self.blogger = self.metaWeblog

        # Create archive root
        if not hasattr(self.aq_inner.aq_explicit, 'archive'):
            # create archive root
            self.portal_types.constructContent('WeblogArchive',
                                               self,
                                               'archive',
                                               title='Archive',
                                               archive_type='root')

        # Create folder to store topic images
        if not hasattr(self.aq_inner.aq_explicit, 'topic_images'):
            self.portal_types.constructContent('Folder',
                                               self,
                                               'topic_images',
                                               title='Topic Images')



    security.declarePrivate('at_post_create_script')
    def at_post_create_script(self): 
        # Setup the default portlets
        if not hasattr(self.aq_inner.aq_explicit, 'left_slots'):
            self.left_slots = DEFAULT_LEFT_SLOTS
        if not hasattr(self.aq_inner.aq_explicit, 'right_slots'):
            self.right_slots = DEFAULT_RIGHT_SLOTS


    # XXX: s/Topics/Categories/
    security.declareProtected(perms.View, 'getTopics')
    def getTopics(self):
        """Return topics/categories instances for this weblog."""
        topics = []
        keywords = self._getKeywordsForBlogEntries()
        for kw in keywords:
            topics.append( Topic(kw).__of__(self) )
        return topics

    # Use getCategories in preference to getTopics
    security.declareProtected(perms.View, 'getCategories')
    getCategories = getTopics

    def _getKeywordsForBlogEntries(self):
        """Return a sequence of all keywords that are associatd with
        IWeblogEntry instances contained in this IWeblog.
        """
        entries = self.getAllEntries()
        # Use dict rather than list to avoid duplicates
        keywords = {}
        for entry in entries:
            for kw in entry.Subject:
                keywords[kw] = None
        return keywords

    # XXX: Fixup for removal of 'archive' folder
    security.declareProtected(perms.View, 'getArchives')
    def getArchives(self):
        """Return WeblogArchive objects for each year."""

        # Currently we have the 'archive' folder only, but this will change
        # soon(tm)
        soon_to_go_away = getattr(self.aq_inner.aq_explicit, 'archive')
        archives = soon_to_go_away.getFolderContents(
                batch=True,
                contentFilter={'portal_type': 'WeblogArchive'},)

        return archives

    security.declareProtected(perms.View, 'getLazyEntries')
    def getLazyEntries(self):
        """Return all entries for this weblog as (lazy) catalog brains.
        """

        archive = getattr(self.aq_inner.aq_explicit, 'archive')
        results = {}
        lazy_entries = archive.getLazyEntries()
        return lazy_entries

    security.declareProtected(perms.ViewDrafts, 'getAllEntries')
    def getAllEntries(self):
        """Return a sequence of *all* IWeblogEntry brain instances within
        this blog.
        """
        path = '/'.join(self.getPhysicalPath())
        results = self.portal_catalog(
                meta_type=['WeblogEntry',],
                path={'query':path, 'level': 0},
                sort_on = 'effective',
                sort_order = 'reverse')
        return results

    # XXX: s/Topics/Categories/ (reall, this will be removed)
    security.declareProtected(perms.View, 'getAvailableTopics')
    def getAvailableTopics(self):
        return self.getTopics()

    security.declareProtected(perms.View, 'getTopicById')
    def getTopicById(self, id):
        """Given a topic id, return an ITopic instance.
        """
        # XXX This should probably check to see if a keyword exists for id.
        return Topic(id).__of__(self)

    security.declareProtected(perms.ViewDrafts, 'getDrafts')
    def getDrafts(self):
        """Return a list of lazy_maps for all WeblogEntries in this Weblog that are
        in the 'draft' review_state."""

        path = '/'.join(self.getPhysicalPath())
        results = self.portal_catalog(
                meta_type=['WeblogEntry',],
                path={'query':path, 'level': 0},
                sort_on = 'effective',
                sort_order = 'reverse',
                review_state = 'draft')

        return results

    security.declareProtected(perms.View, 'getAuthors')
    def getAuthors(self):
        """Returns a sequence of member ids for all members who have
        posted to this Weblog instance.
        """
        path = '/'.join(self.getPhysicalPath())
        results = self.portal_catalog(
            meta_type=['WeblogEntry',],
            path={'query':path, 'level': 0},
            review_state = 'published')
        authors = {}
        for brain in results:
            authors[brain.Creator] = None
        return authors.keys()


registerType(Weblog, PROJECTNAME)
