#
# Authors: Brian Skahan <bskahan@etria.com>
#          Tom von Schwerdtner <tvon@etria.com>
#
# Copyright 2004-2005, Etria, LLP
# # This file is part of Quills
#
# Quills is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Quills is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Quills; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################

# Archetypes imports (with optional Linguaplone integration)
try:
    from Products.LinguaPlone.public import *
except ImportError:
    from Products.Archetypes.public import * 
from Products.Archetypes.Marshall import RFC822Marshaller

# CMF imports
from Products.CMFCore.utils import getToolByName

# Product imports
from config import PROJECTNAME
from interfaces import IWeblogEntry
from topic import Topic
import permissions as perms

# Zope 2 imports
from AccessControl import ClassSecurityInfo

# Zope 3 imports
from zope.interface import implements


WeblogEntrySchema = BaseSchema.copy() + Schema((

    TextField('description',
        searchable=1,
        accessor="Description",
        widget=TextAreaWidget(
            label='Excerpt',
            description="""A brief introduction for this entry.""",
            label_msgid="label_description",
            description_msgid="help_description",
            i18n_domain="quills"),
        ),

    TextField('text',
        searchable=1,
        default_output_type='text/x-html-safe',
        default_content_type = 'text/html',
        allowable_content_types=(
            'text/structured',
            'text/restructured',
            'text/plain',
            'text/html',
            ),
        widget=RichWidget(label='Entry Text',
            rows=30,
            label_msgid="label_text",
            i18n_domain="quills"),
        ),
    ),

    marshall=RFC822Marshaller(),
    )

# Move the subject/topic picking to the main edit view as it should be used
# for every edit, really.
WeblogEntrySchema['subject'].schemata = 'default'
# The subject is not language-specific
WeblogEntrySchema['subject'].languageIndependent = True,
# Make sure it is presented after the main text entry field.
WeblogEntrySchema.moveField('subject', after='text')
# Make sure the allowDiscussion field's default is None
# XXX This appears to work incorrectly at the moment.  The None default
#     seems to translate into True in the UI.  I have no idea why this
#     is getting set to False for instances in the absence of the next line
WeblogEntrySchema['allowDiscussion'].default = None
# Put the discussion setting on the main page...
WeblogEntrySchema['allowDiscussion'].schemata = 'default'
# ... at the bottom, after the subject keywords.
WeblogEntrySchema.moveField('allowDiscussion', after='subject')


class WeblogEntry(BaseContent):
    """Basic Weblog Entry"""

    implements(IWeblogEntry)

    schema = WeblogEntrySchema
    archetype_name = 'Weblog Entry'
    meta_type = 'WeblogEntry'
    content_icon = 'weblogentry_icon.gif'
    global_allow = False
    default_view  = 'weblogentry_view'
    immediate_view = 'weblogentry_view'
    _at_rename_after_creation = True

    security = ClassSecurityInfo()

    actions = (
        { 'id': 'view',
          'name': 'View',
          'action': 'string:${object_url}/weblogentry_view',
          'permissions': (perms.View,) 
        },
        { 'id': 'trackbacks',
          'name': 'Trackbacks',
          'action': 'string:${object_url}/edit_outtrackbacks',
          'permissions': ('plonetrackback: Manage Trackbacks',) 
        },
        )

    # XXX Why are we aliasing 'start' and 'end' to 'effective'?  Can't we just
    # use self.effective() directly?

    # Set start/end to effective to use portal_calendar for a blog archive
    security.declareProtected(perms.View, 'start')
    def start(self):
        return self.effective()

    security.declareProtected(perms.View, 'end')
    def end(self):
        return self.effective()

    security.declareProtected(perms.View, 'getCategories')
    def getCategories(self):
        """See IWeblogEntry.
        """
        qt = getToolByName(self, 'quills_tool')
        weblog = qt.getParentWeblog(self)
        topics = []
        keywords = self.Subject()
        for kw in keywords:
            topics.append( Topic(kw).__of__(weblog) )
        return topics

    # BBB: Alias for backwards compatibility (which we probably don't need)
    #getEntryCategories = getCategories

    security.declareProtected(perms.View, 'getExcerpt')
    def getExcerpt(self):
        """See IWeblogEntry.
        """
        # This is just an alias for Description in this case, but adapters
        # providing IWeblogEntry for other content types may wish to provide
        # a different implementation - say, the first X characters, or the
        # first Y words.
        return self.Description()

registerType(WeblogEntry, PROJECTNAME)
