#
# Authors: Brian Skahan <bskahan@etria.com>
#          Tom von Schwerdtner <tvon@etria.com>
#
# Copyright 2004-2005, Etria, LLP
#
# This file is part of Quills
#
# Quills is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Quills is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Quills; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##############################################################################
"""Migrate from 0.9 to 1.5.
"""

# CMF imports
from Products.CMFCore.utils import getToolByName

# Product imports
from Products.Quills import MetaWeblogAPI

# plonetrackback imports
from Products.plonetrackback.interfaces import ITrackbackOutManager

# Standard lib imports
from StringIO import StringIO


class Migration(object):
    """Migrate from 0.9 to 1.5
    """

    def __init__(self, site, out):
        self.site = site
        self.out = out
        self.catalog = getToolByName(self.site, 'portal_catalog')

    def migrate(self):
        """Run migration on site object passed to __init__.
        """
        print >> self.out, u"Migrating Quills 0.9 -> 1.5"
        self.removeEntryCategoriesIndex()
        # Updating of schemas is handled in the installer
        weblogs = self.catalog(meta_type='Weblog')
        for weblog in weblogs:
            self.migrateWeblog(weblog.getObject())

    def removeEntryCategoriesIndex(self):
        """Remove the getEntryCategories index if present.

        We now use the Subject index, so we can zap it.
        """
        index = 'getEntryCategories'
        if index in self.catalog.indexes():
            self.catalog.manage_delIndex(index)
            print >> self.out, u"Deleted getEntryCategories index"

    def migrateWeblog(self, weblog):
        """Migrate a weblog.
        """
        print >> self.out, \
            u'Migrating Weblog: %s' % weblog.Title()
        self.removeBloggerAPI(weblog)
        self.migrateWeblogTopics(weblog)
        for brain in weblog.getAllEntries():
            entry = brain.getObject()
            self.migrateTrackbacksForEntry(entry)
            self.migrateCategoriesForEntry(entry)

    def removeBloggerAPI(self, weblog):
        """Make sure that BloggerAPI is gone from both the weblog instance
        and its associated RPCAuth instance.
        """
        try:
            delattr(weblog, 'blogger')
        except AttributeError:
            # It's already gone - almost certainly in the 08to09 migration,
            # but there's no harm in being sure ;-).
            pass
        # Now alias metaWeblog to blogger for Performancing compatibility.
        # See <http://performancing.com/node/3158> for more details.
        weblog.blogger = weblog.metaWeblog
        # Define the auth methods that we need to remove from RPCAuth
        authMethods = [
            'blogger/getTemplate',
            'blogger/setTemplate',
            'blogger/deletePost',
            'blogger/editPost',
            'blogger/newPost',
            'blogger/getRecentPosts',
            'blogger/getUserInfo',
            'blogger/getUsersBlogs',
            ]
        quills_tool = getToolByName(weblog, "quills_tool")
        # Find our local RPCAuth instance
        # QuillsTool.findRPCAuth should die, so I don't use it here
        #rpcauth = quills_tool.findRPCAuth(weblog)
        portal_ob = getToolByName(weblog, 'portal_url').getPortalObject()
        rpcauth = portal_ob.RPCAuth
        # Remove the auth methods as they have persistent references to
        # code from the former BloggerAPI module. This doesn't give an
        # error if it has already been called.
        rpcauth.removeAuthProvider(authMethods)
        # Now add back the MetaWeblogAPI auth methods as there are some
        # blogger ones in there.
        rpcauth.addAuthrProvider(MetaWeblogAPI.authMethods)

    def migrateTrackbacksForEntry(self, entry):
        """Swap to multiple trackback URLs stored with an adapter.
        """
        if not hasattr(entry, 'trackbackURLs'):
            # The attribute doesn't exist anymore so nothing to
            # migrate!
            return
        # Adapt to the interface for storing trackback details
        tbim = ITrackbackOutManager(entry)
        # Get the existing URLs by attribute access as the methods are no longer
        # available - the schema no longer defines them.
        urls = getattr(entry, 'trackbackURLs', None)
        if urls:
            #print >> self.out, \
            #    "Migrating trackbacks for %s." % entry.Title()
            wftool = getToolByName(entry, 'portal_workflow')
            review_state = wftool.getInfoFor(entry, 'review_state')
            if review_state == 'published':
                # If the entry is published, the URLs will already have been pinged
                tbim.setPingedURLs(urls)
            else:
                # ... otherwise, they still need to be pinged.
                tbim.setURLsToPing(urls)
        # Remove the attribute.
        del entry.trackbackURLs

    def migrateCategoriesForEntry(self, entry):
        """Migrate the old categories to standard subjects.
        """
        if not hasattr(entry, 'entryCategories'):
            # Nothing to do here.
            return
        # Get the existing categories by direct attribute access as
        # the methods are no longer available - the schema no longer
        # defines them.
        cats = list(entry.entryCategories)
        # Get the existing subject keywords
        existingSubjects = list( entry.Subject() )
        # Add the two together
        subjects = existingSubjects[:] # Hard copy
        subjects.extend(cats)
        # Filter them to avoid duplicates, can happen with multiple
        # calls to the migration or because you already tagged the
        # entry with that keyword.
        uniqueSubjects = {}
        for subject in subjects:
            uniqueSubjects[subject] = 1
        subjects = uniqueSubjects.keys()
        # See if anything changed
        subjects.sort()
        existingSubjects.sort()
        if subjects != existingSubjects:
            # Set the new subjects list
            entry.setSubject(subjects)
            # Update the catalog
            self.catalog.reindexObject(entry)
            #print >> self.out, \
            #    "Migrated categories for '%s' to subjects: %r" % (
            #    entry.Title(),
            #    subjects)
        # Remove the entryCategories attribute
        del entry.entryCategories

    def migrateWeblogTopics(self, weblog):
        """Migrate a weblog topic.
        """
        # XXX Need to disable WeblogTopic content type after migration of topics
        if getattr(weblog.aq_explicit, 'topic_images', None) is None:
            weblog.invokeFactory('Folder', 
                                 'topic_images',
                                 title='Topic Images')
            print >> self.out, "Creted topic_images folder in weblog"
        topic_images = weblog.topic_images
        to_be_deleted = []
        for ob in weblog.objectValues():
            if getattr(ob, 'portal_type', None) == 'WeblogTopic':
                title = ob.Title()
                title = title.replace('/', '_').replace('?', '_')
                img = StringIO(ob.getTopicImage())
                if getattr(topic_images, title, None) is None:
                    # No title, so existing images folder as we
                    # otherwise just created the folder *with* a
                    # title.
                    topic_images.invokeFactory('Image', id=title)
                    newimg = getattr(topic_images, title)
                    newimg.setImage(img)
                    print >> self.out, "Copied topic image."
                to_be_deleted.append(ob.getId())
        weblog.manage_delObjects(ids=to_be_deleted)
