#
# Authors: Brian Skahan <bskahan@etria.com>
#          Tom von Schwerdtner <tvon@etria.com>
#
# Copyright 2004-2005, Etria, LLP
#
# This file is part of Quills
#
# Quills is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# Quills is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Quills; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
###############################################################################

# Standard library imports
from types import StringTypes

# Zope 3 imports
from zope.interface import implements

# Zope 2 imports
from Acquisition import Implicit
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass

# CMF imports
from Products.CMFCore.utils import getToolByName

# Product imports
from interfaces import ITopic
from acquiringactions import AcquiringActionProvider


class Topic(AcquiringActionProvider, Implicit):
    """Implementation of ITopic as a transient wrapper around a simple keyword.
    """

    implements(ITopic)

    security = ClassSecurityInfo()
    security.declareObjectProtected('View')

    def __init__(self, keywords):
        if isinstance(keywords, StringTypes):
            self.keywords = [keywords,]
        else:
            self.keywords = keywords
        # We will cache catalog queries for performance reasons.
        # This should not cost anything as these objects only last for the
        # duration of a REQUEST anyway.
        self.results = None

    security.declareProtected('View', 'getId')
    def getId(self):
        """See ITopic.
        """
        return self.keywords[0]

    security.declareProtected('View', 'getTitle')
    def getTitle(self):
        """See ITopic.
        """
        image = self.getImage()
        if image is not None:
            return image.Title()
        return ''

    security.declareProtected('View', 'title_or_id')
    def title_or_id(self):
        """See ITopic.
        """
        image = self.getImage()
        if image is not None:
            return image.title_or_id()
        return self.getId()

    security.declareProtected('View', 'getDescription')
    def getDescription(self):
        """See ITopic.
        """
        image = self.getImage()
        if image is not None:
            return image.Description()
        return ''

    security.declareProtected('View', 'getImage')
    def getImage(self):
        """See ITopic.
        """
        topic_images = getattr(self.aq_parent.aq_explicit, 'topic_images', None)
        if topic_images is not None:
            hasimage = getattr(topic_images.aq_explicit, self.keywords[0], None)
            if hasimage is not None:
                # Ugh! Acquisition sucks. Can't use the hasimage variable as
                # zope/plone expects an implicit acquisition context, not an
                # explicit one.
                return getattr(topic_images, self.keywords[0])
        # Just to be explicit...
        return None

    security.declareProtected('View', 'getLazyEntries')
    def getLazyEntries(self):
        """See ITopic.
        """
        # Try to return the cached query if possible
        if self.results is not None:
            return self.results
        weblog = self.quills_tool.getParentWeblog(self)
        path = '/'.join(weblog.getPhysicalPath())
        self.results = self.portal_catalog(meta_type='WeblogEntry',
                path={'query':path, 'level': 0},
                Subject = { 'query' : self.keywords,
                            'operator' : 'and'},
                sort_on = 'effective',
                sort_order = 'reverse',
                review_state = 'published')
        return self.results

    def __str__(self):
        """String representation of the Topic"""
        return self.keywords[0]

    def __len__(self):
        """Return the number of posts in this topic.
        """
        # Make sure we've got the results
        self.getLazyEntries()
        return len(self.results)

    def getLastModified(self):
        """See ITopic.
        """
        # Make sure we've got the results for this topic
        self.getLazyEntries()
        if self.results:
            return self.results[0].modified
        else:
            # Just to be explicit about it
            return None


class AuthorsTopic(Topic):
    """
    """

    def getTitle(self):
        """See ITopic.
        """
        memb_tool = getToolByName(self, 'portal_membership')
        return memb_tool.getMemberInfo(self.keywords[0])['fullname']


    def title_or_id(self):
        """See ITopic.
        """
        memb_tool = getToolByName(self, 'portal_membership')
        title = memb_tool.getMemberInfo(self.keywords[0])['fullname']
        if title:
            return title
        return self.getId()

    def getDescription(self):
        """See ITopic.
        """
        # XXX Fix me with something sensible.
        # Get the description from the member object?
        return ''

    def getImage(self):
        """See ITopic.
        """
        memb_tool = getToolByName(self, 'portal_membership')
        return memb_tool.getPersonalPortrait(self.keywords[0])

    def getLazyEntries(self):
        """See ITopic.
        """
        weblog = self.quills_tool.getParentWeblog(self)
        path = '/'.join(weblog.getPhysicalPath())
        results = self.portal_catalog(meta_type='WeblogEntry',
                path={'query':path, 'level': 0},
                Creator = { 'query' : self.keywords,
                            'operator' : 'or'},
                sort_on = 'effective',
                sort_order = 'reverse',
                review_state = 'published')
        return results


InitializeClass(Topic)
InitializeClass(AuthorsTopic)